/**
 * @file
 * VuoList_VuoMouseButton C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoMouseButton_H
#define VuoList_VuoMouseButton_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoMouseButton VuoList_VuoMouseButton
 * A list of @ref VuoMouseButton elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoMouseButton_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoMouseButton elements.
 */
typedef const struct VuoList_VuoMouseButton_struct { void *l; } * VuoList_VuoMouseButton;
/// @}
#endif

/**
 * Creates a new list of @ref VuoMouseButton elements.
 */
VuoList_VuoMouseButton VuoListCreate_VuoMouseButton(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoMouseButton to quickly initialize a large list.
 */
VuoList_VuoMouseButton VuoListCreateWithCount_VuoMouseButton(const unsigned long count, const VuoMouseButton value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoMouseButton VuoListCopy_VuoMouseButton(const VuoList_VuoMouseButton list);

/**
 * Returns the @ref VuoMouseButton at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoMouseButton VuoListGetValue_VuoMouseButton(const VuoList_VuoMouseButton list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoMouseButton *VuoListGetData_VuoMouseButton(const VuoList_VuoMouseButton list);

/**
 * Changes the @ref VuoMouseButton at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoMouseButton(const VuoList_VuoMouseButton list, const VuoMouseButton value, const unsigned long index);

/**
 * Inserts the `VuoMouseButton` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoMouseButton(const VuoList_VuoMouseButton list, const VuoMouseButton value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoMouseButton(VuoList_VuoMouseButton list, const VuoMouseButton value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoMouseButton(VuoList_VuoMouseButton list, const VuoMouseButton value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoMouseButton(VuoList_VuoMouseButton list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoMouseButton(VuoList_VuoMouseButton list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoMouseButton(VuoList_VuoMouseButton list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoMouseButton(VuoList_VuoMouseButton list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoMouseButton(VuoList_VuoMouseButton list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoMouseButton(VuoList_VuoMouseButton list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoMouseButton(VuoList_VuoMouseButton list);

/**
 * Removes the `VuoMouseButton` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoMouseButton(VuoList_VuoMouseButton list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoMouseButton(const VuoList_VuoMouseButton list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoMouseButton VuoList_VuoMouseButton_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoMouseButton_getJson(const VuoList_VuoMouseButton value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoMouseButton_getSummary(const VuoList_VuoMouseButton value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoMouseButton VuoList_VuoMouseButton_makeFromString(const char *str);
char * VuoList_VuoMouseButton_getString(const VuoList_VuoMouseButton value);
/// @}

/**
 * @}
 */

#endif
