/**
 * @file
 * VuoList_VuoRssItem C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoRssItem_H
#define VuoList_VuoRssItem_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoRssItem VuoList_VuoRssItem
 * A list of @ref VuoRssItem elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoRssItem_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoRssItem elements.
 */
typedef const struct VuoList_VuoRssItem_struct { void *l; } * VuoList_VuoRssItem;
/// @}
#endif

/**
 * Creates a new list of @ref VuoRssItem elements.
 */
VuoList_VuoRssItem VuoListCreate_VuoRssItem(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoRssItem to quickly initialize a large list.
 */
VuoList_VuoRssItem VuoListCreateWithCount_VuoRssItem(const unsigned long count, const VuoRssItem value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoRssItem VuoListCopy_VuoRssItem(const VuoList_VuoRssItem list);

/**
 * Returns the @ref VuoRssItem at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoRssItem VuoListGetValue_VuoRssItem(const VuoList_VuoRssItem list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoRssItem *VuoListGetData_VuoRssItem(const VuoList_VuoRssItem list);

/**
 * Changes the @ref VuoRssItem at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoRssItem(const VuoList_VuoRssItem list, const VuoRssItem value, const unsigned long index);

/**
 * Inserts the `VuoRssItem` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoRssItem(const VuoList_VuoRssItem list, const VuoRssItem value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoRssItem(VuoList_VuoRssItem list, const VuoRssItem value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoRssItem(VuoList_VuoRssItem list, const VuoRssItem value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoRssItem(VuoList_VuoRssItem list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoRssItem(VuoList_VuoRssItem list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoRssItem(VuoList_VuoRssItem list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoRssItem(VuoList_VuoRssItem list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoRssItem(VuoList_VuoRssItem list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoRssItem(VuoList_VuoRssItem list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoRssItem(VuoList_VuoRssItem list);

/**
 * Removes the `VuoRssItem` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoRssItem(VuoList_VuoRssItem list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoRssItem(const VuoList_VuoRssItem list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoRssItem VuoList_VuoRssItem_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoRssItem_getJson(const VuoList_VuoRssItem value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoRssItem_getSummary(const VuoList_VuoRssItem value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoRssItem VuoList_VuoRssItem_makeFromString(const char *str);
char * VuoList_VuoRssItem_getString(const VuoList_VuoRssItem value);
/// @}

/**
 * @}
 */

#endif
