/**
 * @file
 * VuoList_VuoUrl C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoUrl_H
#define VuoList_VuoUrl_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoUrl VuoList_VuoUrl
 * A list of @ref VuoUrl elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoUrl_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoUrl elements.
 */
typedef const struct VuoList_VuoUrl_struct { void *l; } * VuoList_VuoUrl;
/// @}
#endif

/**
 * Creates a new list of @ref VuoUrl elements.
 */
VuoList_VuoUrl VuoListCreate_VuoUrl(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoUrl to quickly initialize a large list.
 */
VuoList_VuoUrl VuoListCreateWithCount_VuoUrl(const unsigned long count, const VuoUrl value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoUrl VuoListCopy_VuoUrl(const VuoList_VuoUrl list);

/**
 * Returns the @ref VuoUrl at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoUrl VuoListGetValue_VuoUrl(const VuoList_VuoUrl list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoUrl *VuoListGetData_VuoUrl(const VuoList_VuoUrl list);

/**
 * Changes the @ref VuoUrl at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoUrl(const VuoList_VuoUrl list, const VuoUrl value, const unsigned long index);

/**
 * Inserts the `VuoUrl` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoUrl(const VuoList_VuoUrl list, const VuoUrl value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoUrl(VuoList_VuoUrl list, const VuoUrl value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoUrl(VuoList_VuoUrl list, const VuoUrl value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoUrl(VuoList_VuoUrl list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoUrl(VuoList_VuoUrl list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoUrl(VuoList_VuoUrl list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoUrl(VuoList_VuoUrl list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoUrl(VuoList_VuoUrl list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoUrl(VuoList_VuoUrl list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoUrl(VuoList_VuoUrl list);

/**
 * Removes the `VuoUrl` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoUrl(VuoList_VuoUrl list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoUrl(const VuoList_VuoUrl list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoUrl VuoList_VuoUrl_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoUrl_getJson(const VuoList_VuoUrl value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoUrl_getSummary(const VuoList_VuoUrl value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoUrl VuoList_VuoUrl_makeFromString(const char *str);
char * VuoList_VuoUrl_getString(const VuoList_VuoUrl value);
/// @}

/**
 * @}
 */

#endif
