/**
 * @file
 * VuoList_VuoWrapMode C type definition.
 *
 * @copyright Copyright © 2012–2015 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoWrapMode_H
#define VuoList_VuoWrapMode_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoWrapMode VuoList_VuoWrapMode
 * A list of @ref VuoWrapMode elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoWrapMode_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoWrapMode elements.
 */
typedef const struct VuoList_VuoWrapMode_struct { void *l; } * VuoList_VuoWrapMode;
/// @}
#endif

/**
 * Creates a new list of @ref VuoWrapMode elements.
 */
VuoList_VuoWrapMode VuoListCreate_VuoWrapMode(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoWrapMode to quickly initialize a large list.
 */
VuoList_VuoWrapMode VuoListCreateWithCount_VuoWrapMode(const unsigned long count, const VuoWrapMode value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoWrapMode VuoListCopy_VuoWrapMode(const VuoList_VuoWrapMode list);

/**
 * Returns the @ref VuoWrapMode at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoWrapMode VuoListGetValue_VuoWrapMode(const VuoList_VuoWrapMode list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoWrapMode *VuoListGetData_VuoWrapMode(const VuoList_VuoWrapMode list);

/**
 * Changes the @ref VuoWrapMode at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoWrapMode(const VuoList_VuoWrapMode list, const VuoWrapMode value, const unsigned long index);

/**
 * Inserts the `VuoWrapMode` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoWrapMode(const VuoList_VuoWrapMode list, const VuoWrapMode value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoWrapMode(VuoList_VuoWrapMode list, const VuoWrapMode value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoWrapMode(VuoList_VuoWrapMode list, const VuoWrapMode value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoWrapMode(VuoList_VuoWrapMode list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoWrapMode(VuoList_VuoWrapMode list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoWrapMode(VuoList_VuoWrapMode list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoWrapMode(VuoList_VuoWrapMode list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoWrapMode(VuoList_VuoWrapMode list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoWrapMode(VuoList_VuoWrapMode list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoWrapMode(VuoList_VuoWrapMode list);

/**
 * Removes the `VuoWrapMode` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoWrapMode(VuoList_VuoWrapMode list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoWrapMode(const VuoList_VuoWrapMode list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoWrapMode VuoList_VuoWrapMode_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoWrapMode_getJson(const VuoList_VuoWrapMode value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoWrapMode_getSummary(const VuoList_VuoWrapMode value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoWrapMode VuoList_VuoWrapMode_makeFromString(const char *str);
char * VuoList_VuoWrapMode_getString(const VuoList_VuoWrapMode value);
/// @}

/**
 * @}
 */

#endif
